/*
	\author Matt English
	\date 9/20/06
	\brief
		This file is the form for the new level form created after clicking the new level button

		All content copyright  2006 DigiPen(USA) Corporation
*/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.DirectX;
using D3D = Microsoft.DirectX.Direct3D;
using DI = Microsoft.DirectX.DirectInput;

namespace ns_Controls
{
    class Camera
    {
        public Camera() { UpdateSpherical(); UpdatePoints(); }
        public Camera(Vector3 eye, Vector3 target, Vector3 up)
        {
            eyePos = eye;
            targetPos = target;
            this.up = up;
            UpdateSpherical();
            UpdatePoints();
        }
        private Vector3 eyePos;

        public Vector3 EyePos
        {
            get { return eyePos; }
            set { eyePos = value; }
        }
        private Vector3 targetPos;

        public Vector3 TargetPos
        {
            get { return targetPos; }
            set { targetPos = value; }
        }
        private Vector3 up;

        public Vector3 Up
        {
            get { return up; }
            set { up = value; }
        }
        // The distance to the origin
        private float radius;

        public float Radius
        {
            get { return radius; }
            set { radius = value; }
        }
        // The azimuth angle [0,2pi]
        private float azimuth;

        public float Azimuth
        {
            get { return azimuth; }
            set { azimuth = value; }
        }
        // The latitude angle [-pi/2,pi/2]
        private float latitude;

        public float Latitude
        {
            get { return latitude; }
            set { latitude = value; }
        }

        public void SetPosition(Vector3 Pos)
        {
	        eyePos = Pos;
            UpdateSpherical();
        }

        public void SetTarget(Vector3 Tar)
        {
	        targetPos = Tar;
            UpdateSpherical();
        }

        public void Move( Vector3 P )
        {
	        Vector3 v = targetPos - eyePos;
	        v.Normalize();
	        Vector3 u = Vector3.Cross(v, up);
            Vector3 w = Vector3.Cross(u, v);

	        eyePos += (v * P.Y) + (u * P.X) + (w * P.Z);

            UpdatePoints();
        }

        public void Turn(int DX, int DY)
        {
	        float yaw = (float)DX/100.0f;
	        float pitch = (float)DY/100.0f;
            Azimuth = (Azimuth + yaw);
            Latitude = (Latitude - pitch);
            UpdatePoints();
        }

        public void UpdateSpherical()
        {
	        Vector3 p = targetPos - eyePos;
	        Radius = p.Length();
	        Latitude = (float)System.Math.Asin(p.Z / Radius);
	        Azimuth = (float)System.Math.Atan2(p.Y, p.Z);
        }

        public void UpdatePoints()
        {
            targetPos = eyePos +  new Vector3((-Radius) * (float)(Math.Cos(Latitude)) * (float)(Math.Cos(Azimuth)),
                                                       Radius * (float)(Math.Cos(Latitude)) * (float)(Math.Sin(Azimuth)),
                                                       Radius * (float)(Math.Sin(Latitude)));
        }
    };
}
