/*
	\author Matt English
	\date 9/5/06
	\brief
		This file contains all the functions relating to the form controls

		All content copyright  2006 DigiPen(USA) Corporation
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;
using lvlformat;
using ns_Controls;

namespace DirectX_Tutorial
{
    public partial class WinForm : System.Windows.Forms.Form
    {
        private NewLevel nlForm;
        private bool SpawnAreaButtonStatus;
        private bool CastleButtonStatus;

        /*!
           Saves the current level.  Called when the save button is clicked
         * 
         * \param sender
         *      the originator form
         * 
         * \param e
         *      Events of the button click

           \author
               Matt English
       */
        private void Button_Save_Click(object sender, EventArgs e)
        {
            int i = 0, j = 0, k=0;
            Stream myStream;
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            SpawnAreaButtonStatus = false;
            CastleButtonStatus = false;


            saveFileDialog1.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*";
            saveFileDialog1.FilterIndex = 2;
            saveFileDialog1.RestoreDirectory = true;
            saveFileDialog1.OverwritePrompt = false;
            saveFileDialog1.CreatePrompt = false;

            DialogResult res = saveFileDialog1.ShowDialog();
            if (res == DialogResult.OK || res == DialogResult.Yes)
            {
                if ((myStream = saveFileDialog1.OpenFile()) != null)
                {
                    // Code to write the stream goes here.
                    byte[] b = new byte[4];
                    //convert and write the width
                    for (i = 0; i < sizeof(Int32); ++i)
                    {
                        b[i] = (byte)((curLevel.width >> (i * 8)) & 0x000000FF);
                    }
                    myStream.Write(b, 0, sizeof(Int32));
                    //convert and write the height
                    for (i = 0; i < sizeof(Int32); ++i)
                    {
                        b[i] = (byte)((curLevel.height >> (i * 8)) & 0x000000FF);
                    }
                    myStream.Write(b, 0, sizeof(Int32));
                    byte[] newb = new byte[curLevel.width * curLevel.height * sizeof(Int32)];
                    /*
                     * convert and write the heightmap data
                     * Will be written row by row
                     * */
                    for (i = 0; i < curLevel.height; ++i)
                    {
                        for (j = 0; j < curLevel.width * 4; j+=4)
                        {
                            for (k = 0; k < sizeof(Int32); ++k)
                            {
                                newb[(i * curLevel.width * 4) + j + k] = (byte)((curLevel.heightMap[i, j / sizeof(Int32)] >> (k * 8)) & 0x000000FF);
                                //newb[(i * curLevel.width) + j + k] = (byte)((curLevel.heightMap[i, j / sizeof(Int32)] >> (k * 8)) & 0x000000FF);
                            }
                        }
                    }
                    myStream.Write(newb, 0, sizeof(Int32) * curLevel.width * curLevel.height);
                    //convert and write the number of spawn vertices
                    for (i = 0; i < sizeof(Int32); ++i)
                    {
                        b[i] = (byte)((curLevel.numOfSpawnVerts >> (i * 8)) & 0x000000FF);
                    }
                    myStream.Write(b, 0, sizeof(Int32));
                    //convert and write the peasant spawn locations
                    newb = new byte[curLevel.numOfSpawnVerts * sizeof(Int32)];
                    for (i = 0; i < curLevel.numOfSpawnVerts; ++i)
                    {
                        int temp = (int)curLevel.spawnVertices[i];
                        for (k = 0; k < sizeof(Int32); ++k)
                        {
                            newb[(i * 4) + k] = (byte)((temp >> (k * 8)) & 0x000000FF);
                        }
                    }
                    myStream.Write(newb, 0, sizeof(Int32) * curLevel.numOfSpawnVerts);
                    //whether the castle has been saved
                    newb = new byte[sizeof(bool)];
                    if (curLevel.castleSpawned == true)
                        newb[0] = 1;
                    else
                        newb[0] = 0;
                    myStream.Write(newb, 0, sizeof(bool));
                    //now if the castle has spawned write it's location
                    if (curLevel.castleSpawned == true)
                    {
                        newb = new byte[sizeof(float)];
                        newb = System.BitConverter.GetBytes(curLevel.castleLoc.X);
                        myStream.Write(newb, 0, sizeof(float));
                        newb = System.BitConverter.GetBytes(curLevel.castleLoc.Y);
                        myStream.Write(newb, 0, sizeof(float));
                        newb = System.BitConverter.GetBytes(curLevel.castleLoc.Y);
                        myStream.Write(newb, 0, sizeof(float));
                    }
                    myStream.Close();
                }
            }
        }//end of save

        /*!
           Launches a load file dialog box .  Called when the load button is clicked, and will load the file
         * selected by the user
         * 
         * \param sender
         *      the originator form
         * 
         * \param e
         *      Events of the button click

           \author
               Matt English
       */
        private void Load_Button_Click(object sender, EventArgs e)
        {
            Stream loadStream;
            int i = 0, j = 0;
            int readPointer = 0;
            byte[] b = new byte[sizeof(Int32)];
            OpenFileDialog openFile = new OpenFileDialog();
            openFile.FilterIndex = 2;
            openFile.RestoreDirectory = true;
            DialogResult res = openFile.ShowDialog();

            SpawnAreaButtonStatus = false;
            CastleButtonStatus = false;


            if (res == DialogResult.OK || res == DialogResult.Yes)
            {
                if ((loadStream = openFile.OpenFile()) != null)
                {
                    loadStream.Read(b, 0, sizeof(Int32));
                    //make sure to increment the read pointer after after read op
                    //readPointer += sizeof(Int32);
                    curLevel.width = System.BitConverter.ToInt32(b, 0);
                    loadStream.Read(b, 0, sizeof(Int32));
                    //readPointer += sizeof(Int32);
                    curLevel.height = System.BitConverter.ToInt32(b, 0);
                    //set the heightmap and byte array to the new  width and height
                    curLevel.heightMap = new int[curLevel.width, curLevel.height];
                    byte[] newb = new byte[curLevel.width * curLevel.height * sizeof(Int32)];
                    //resize the lvl
                    curLevel = new clvlformat(curLevel.width, curLevel.height);
                    loadStream.Read(newb, 0, sizeof(Int32) * curLevel.width * curLevel.height);
                    //convert all the byte data to ints
                    for (i = 0; i < curLevel.height; ++i)
                    {
                        for (j = 0; j < curLevel.width * 4; j+=4)
                        {
                            curLevel.heightMap[i, j / sizeof(Int32)] = System.BitConverter.ToInt32(newb, (i * curLevel.width * sizeof(Int32)) + j);
                        }
                    }
                    //read in the number of spawn vertices
                    loadStream.Read(b, 0, sizeof(Int32));
                    curLevel.numOfSpawnVerts = System.BitConverter.ToInt32(b, 0);
                    //now read in the spawn vertices
                    newb = new byte[curLevel.numOfSpawnVerts * sizeof(Int32)];
                    loadStream.Read(newb, 0, sizeof(Int32) * curLevel.numOfSpawnVerts);
                    for(i=0; i < curLevel.numOfSpawnVerts; ++i)
                    {
                        curLevel.spawnVertices.Add(System.BitConverter.ToInt32(newb, i * sizeof(Int32)));
                    }
                    //now read in whether there is a castle
                    b = new byte[sizeof(bool)];
                    loadStream.Read(b, 0, sizeof(bool));
                    curLevel.castleSpawned = System.BitConverter.ToBoolean(b, 0);
                    if (curLevel.castleSpawned)
                    {
                        //read in the castle locations
                        b = new byte[sizeof(float)];
                        loadStream.Read(b, 0, sizeof(float));
                        curLevel.castleLoc.X = System.BitConverter.ToSingle(b, 0);
                        loadStream.Read(b, 0, sizeof(float));
                        curLevel.castleLoc.Y = System.BitConverter.ToSingle(b, 0);
                        loadStream.Read(b, 0, sizeof(float));
                        curLevel.castleLoc.Z = System.BitConverter.ToSingle(b, 0);
                    }
                    this.VertexDeclaration();
                    this.IndiciesDeclaration();
                }
            }//end of openfile check
        }//end of load button click
        /*!
           Loads the new level form
         * 
         * \param sender
         *      the originator form
         * 
         * \param e
         *      Events of the button click

           \author
               Matt English
       */
        private void Button_NewLevel_Click(object sender, EventArgs e)
        {
            curLevel.numOfSpawnVerts = 0;
            curLevel.spawnVertices.Clear();
            curLevel.castleSpawned = false;
            SpawnAreaButtonStatus = false;
            CastleButtonStatus = false;
			//System.Windows.Forms.MessageBox.Show(this, "Please", "Caption", System.Windows.Forms.MessageBoxButtons.OK);
            nlForm = new NewLevel(this);
			//System.Windows.Forms.MessageBox.Show(this, "Please", "Caption", System.Windows.Forms.MessageBoxButtons.OK);
			//MessageBox.Show(this, "Please enter a width greater than 0 and below 1024", "Caption", MessageBoxButtons.OK);
            nlForm.ShowDialog(this);
        }//end of new level button click
        /*!
           Sets the SpawnnAreaButtonStatus
         * 
         * \param sender
         *      the originator form
         * 
         * \param e
         *      Events of the button click

           \author
               Matt English
       */
        private void SpawnArea_Button_Click(object sender, EventArgs e)
        {
            if (SpawnAreaButtonStatus == false)
                SpawnAreaButtonStatus = true;
            else
                SpawnAreaButtonStatus = false;
        }
        /*!
           CastleButtonStatus
         * 
         * \param sender
         *      the originator form
         * 
         * \param e
         *      Events of the button click

           \author
               Matt English
       */
        private void Castle_Button_Click(object sender, EventArgs e)
        {
            SpawnAreaButtonStatus = false;
            if (CastleButtonStatus == false)
                CastleButtonStatus = true;
            else
                CastleButtonStatus = false;
        }
    }
}
